package com.base22.wcm.plugin.rendering;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;

import com.ibm.portal.ListModel;
import com.ibm.portal.ModelException;
import com.ibm.workplace.wcm.api.DocumentId;
import com.ibm.workplace.wcm.api.DocumentIdIterator;
import com.ibm.workplace.wcm.api.DocumentLibrary;
import com.ibm.workplace.wcm.api.DocumentTypes;
import com.ibm.workplace.wcm.api.LibraryHTMLComponent;
import com.ibm.workplace.wcm.api.WCM_API;
import com.ibm.workplace.wcm.api.Workspace;
import com.ibm.workplace.wcm.api.exceptions.AuthorizationException;
import com.ibm.workplace.wcm.api.exceptions.DocumentRetrievalException;
import com.ibm.workplace.wcm.api.exceptions.OperationFailedException;
import com.ibm.workplace.wcm.api.exceptions.ServiceNotAvailableException;
import com.ibm.workplace.wcm.api.plugin.rendering.RenderingPlugin;
import com.ibm.workplace.wcm.api.plugin.rendering.RenderingPluginException;
import com.ibm.workplace.wcm.api.plugin.rendering.RenderingPluginModel;

/**
 * A rendering plugin that integrates the Velocity Template Engine with WCM.
 */
public class VelocityPlugin implements RenderingPlugin {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public static final String CMPNT_NAME = "cmpnt";
	public static final String LIBRARY_NAME = "lib";
	public static final String CONTEXT_VARS = "vars";

	/**
	 * A simple list model holding locales.
	 */
	protected static class SimpleLocaleListModel<K> implements ListModel<Locale> {

		/** the list of locales of this list model */
		final List<Locale> m_localeList = new ArrayList<Locale>();

		/**
		 * Constructs this simple list model holding the given locales.
		 * 
		 * @param locales
		 *            the locales of this list model. May be <code>null</code>.
		 */
		public SimpleLocaleListModel(final Locale[] p_locales) {
			if (p_locales != null) {
				for (int i = 0; i < p_locales.length; ++i) {
					m_localeList.add(p_locales[i]);
				}
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see com.ibm.portal.ListModel#iterator()
		 */
		@Override
		public Iterator<Locale> iterator() throws ModelException {
			return m_localeList.iterator();
		}
	}

	/** a list model that only contains the English language locale */
	private static final ListModel<Locale> ENGLISH_ONLY = new SimpleLocaleListModel<Locale>(new Locale[] { Locale.ENGLISH });

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.portal.Localized#getDescription(java.util.Locale)
	 */
	@Override
	public String getDescription(final Locale p_locale) {
		return "Provides ability to use Velocity Templates in WCM.";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.portal.Localized#getLocales()
	 */
	@Override
	public ListModel<Locale> getLocales() {
		return ENGLISH_ONLY;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.workplace.wcm.api.plugin.rendering#getName()
	 */
	@Override
	public String getName() {
		return "Velocity";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.portal.Localized#getTitle(java.util.Locale)
	 */
	@Override
	public String getTitle(final Locale p_locale) {
		return "Velocity";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ibm.workplace.wcm.api.plugin.AuthoringPlugin#isShownInAuthoringUI()
	 */
	@Override
	public boolean isShownInAuthoringUI() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.workplace.wcm.api.plugin.rendering.RenderingPlugin#
	 * render(com.ibm.workplace.wcm.api .plugin.rendering.RenderingPluginModel)
	 */
	@Override
	public boolean render(final RenderingPluginModel p_model) throws RenderingPluginException {

		HttpServletRequest request = (HttpServletRequest) p_model.getRequest();

		final Map<String, List<String>> params = p_model.getPluginParameters();
		final Writer writer = p_model.getWriter();

		String htmlCmpntName = null;
		String contextVariables = null;

		try {

			if (!params.containsKey(CMPNT_NAME)) {
				writer.write("<br/><span style=\"color:red\">WCM Velocity plugin parameter '" + CMPNT_NAME + "' is required, but has either been omitted or mispelled.</span><br/>");
			}
			if (!params.containsKey(LIBRARY_NAME)) {
				writer.write("<br/><span style=\"color:red\">WCM Velocity plugin parameter '" + LIBRARY_NAME + "' is required, but has either been omitted or mispelled. ");
				writer.write("Provide the name of the WCM library in which the HTML component can be found.</span><br/>");
			}

			if (params.containsKey(CMPNT_NAME) && params.containsKey(LIBRARY_NAME)) {

				htmlCmpntName = params.get(CMPNT_NAME).get(0);

				Workspace workspace = WCM_API.getRepository().getSystemWorkspace();
				// Workspace workspace = WCM_API.getRepository().getWorkspace(request.getUserPrincipal());
				
				DocumentLibrary library = workspace.getDocumentLibrary(LIBRARY_NAME);
				if (library != null) {
					workspace.setCurrentDocumentLibrary(library);
				}
				
				DocumentIdIterator docIds = workspace.findByName(DocumentTypes.HTMLComponent, htmlCmpntName);

				if (docIds.hasNext()) {

					DocumentId docId = (DocumentId) docIds.next();

					

					LibraryHTMLComponent cmpnt = (LibraryHTMLComponent) workspace.getById(docId);
					String rawTemplate = workspace.render(p_model.getRenderingContext(), cmpnt);

					/* Init the runtime engine. Defaults are fine. */
					Velocity.init();

					VelocityContext context = new VelocityContext();

					if (params.containsKey(CONTEXT_VARS)) {
						contextVariables = params.get(CONTEXT_VARS).get(0);

						if (contextVariables.contains("|") && contextVariables.contains("=")) {
							String[] keyValPairs = contextVariables.split("\\|");
							for (int i = 0; i < keyValPairs.length; i++) {
								String pair = keyValPairs[i];
								if (pair.contains("=")) {
									StringTokenizer st = new StringTokenizer(pair, "=");
									while (st.hasMoreTokens()) {
										String key = st.nextToken().trim();
										String val = st.nextToken().trim();
										context.put(key, val);
									}
								}
							}
						}
						else if (contextVariables.contains("=")) {
							StringTokenizer st = new StringTokenizer(contextVariables, "=");
							while (st.hasMoreTokens()) {
								String key = st.nextToken().trim();
								String val = st.nextToken().trim();
								context.put(key, val);
							}
						}

					}

					StringWriter w = new StringWriter();
					Velocity.evaluate(context, w, "mystring", rawTemplate);
					writer.write(w.toString());

				}

			}

		} catch (ServiceNotAvailableException e) {
			e.printStackTrace();
		} catch (OperationFailedException e) {
			e.printStackTrace();
		} catch (AuthorizationException e) {
			e.printStackTrace();
		} catch (DocumentRetrievalException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		// determine whether inner contents of the plugin should be rendered
		final boolean renderBody;

		final List<String> renderBodyList = params.get("renderbody");

		if (renderBodyList != null && renderBodyList.get(0).equals("false")) {
			renderBody = false;
		}
		else {
			renderBody = true;
		}

		return renderBody;
	}

	public String getRenderedTemplate() {

		/* first, we init the runtime engine. Defaults are fine. */
		Velocity.init();

		/* lets make a Context and put data into it */
		VelocityContext context = new VelocityContext();
		context.put("name", "Velocity");
		context.put("project", "Jakarta");

		StringWriter w = new StringWriter();
		String s = "We are using $project $name to render this.";
		Velocity.evaluate(context, w, "mystring", s);

		return w.toString();
	}

	public void testContextParsing() {

		System.out.println("testContextParsing");

		// String contextVariables = "name = Velocity | project=Jakarta";
		String contextVariables = "name = Velocity";

		if (contextVariables.contains("|") && contextVariables.contains("=")) {
			String[] keyValPairs = contextVariables.split("\\|");
			for (int i = 0; i < keyValPairs.length; i++) {
				String pair = keyValPairs[i];
				if (pair.contains("=")) {
					StringTokenizer st = new StringTokenizer(pair, "=");
					while (st.hasMoreTokens()) {
						String key = st.nextToken().trim();
						String val = st.nextToken().trim();
						System.out.println("Key: " + key + " | " + "Val: " + val);
					}
				}
			}
		}
		else if (contextVariables.contains("=")) {
			StringTokenizer st = new StringTokenizer(contextVariables, "=");
			while (st.hasMoreTokens()) {
				String key = st.nextToken().trim();
				String val = st.nextToken().trim();
				System.out.println("Key: " + key + " | " + "Val: " + val);
			}
		}
	}

	/**
	 * A main method for convenient console testing...
	 */
	public static void main(String[] args) {
		VelocityPlugin plugin = new VelocityPlugin();
		plugin.testContextParsing();
		// System.out.println( plugin.getRenderedTemplate() );
	}

}
